      SUBROUTINE PDLACPT3( UPLO, M, N, A, IA, JA, DESCA, B, IB, JB,
     $                     DESCB )
*
*
*  -- ScaLAPACK auxiliary routine (version 2.0) --
*     University of Tennessee, Knoxville, Oak Ridge National Laboratory,
*     and University of California, Berkeley.
*     Oct 10, 1996
*
*
*    Purpose
*    =======
*
*    Copy a vector from a dense matrix
*    sub(A) into sub(B) in packed storage
*    where
*
*    sub(A) = A(ia:ia+m-1,ja:ja+n-1)
*    sub(B) = B(ib:ib+m-1,jb:jb+n-1)
*
*
*
*     .. Parameters ..
      INTEGER            DLEN_
      PARAMETER          ( DLEN_ = 9 )
      INTEGER            CTXT_, MB_, NB_
      PARAMETER          ( CTXT_ = 2, MB_ = 5, NB_ = 6 )
      INTEGER            RSRC_, CSRC_, LLD_
      PARAMETER          ( RSRC_ = 7, CSRC_ = 8, LLD_ = 9 )
*     ..
*     .. Scalar Arguments ..
      CHARACTER          UPLO
      INTEGER            IA, IB, JA, JB, M, N
*     ..
*     .. Array Arguments ..
      INTEGER            DESCA( * ), DESCB( * )
      DOUBLE PRECISION   A( * ), B( * )
*     ..
*     .. Local Scalars ..
      LOGICAL            ALL_LOCAL, HASWORK
      INTEGER            COFFA, COFFB, CPROC, CSRC, IAROW, IBEND, IBROW,
     $                   IBSTART, IDXA, IDXB, IIB, ISIZE, J, JACOL,
     $                   JBCOL, JEND, JJB, JSIZE, JSTART, LCINDX, LDA,
     $                   LDB, LOFFSET, LRINDX, MB, MYPCOL, MYPROW, NB,
     $                   NPCOL, NPROW, ROFFA, ROFFB, RSRC
*     ..
*     .. Local Arrays ..
      INTEGER            DESCNEW( DLEN_ )
*     ..
*     .. External Functions ..
      INTEGER            INDXFIRST, INDXG2P, NUMROC2
      EXTERNAL           INDXFIRST, INDXG2P, NUMROC2
*     ..
*     .. External Subroutines ..
      EXTERNAL           BLACS_GRIDINFO, DCOPY, DESCINITT, INFOG2L,
     $                   PDCOPY
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          MAX, MIN, MOD
*     ..
*     .. Executable Statements ..
      NB = DESCA( NB_ )
      MB = DESCA( MB_ )
      CALL BLACS_GRIDINFO( DESCA( CTXT_ ), NPROW, NPCOL, MYPROW,
     $                     MYPCOL )
*
*  Check for special case.
*
      ALL_LOCAL = .false.
      IF( DESCA( CTXT_ ).EQ.DESCB( CTXT_ ) ) THEN
         ROFFA = MOD( IA-1, DESCA( MB_ ) )
         COFFA = MOD( JA-1, DESCA( NB_ ) )
         ROFFB = MOD( IB-1, DESCB( MB_ ) )
         COFFB = MOD( JB-1, DESCB( NB_ ) )
         IAROW = INDXG2P( IA, DESCA( MB_ ), MYPROW, DESCA( RSRC_ ),
     $           NPROW )
         JACOL = INDXG2P( JA, DESCA( NB_ ), MYPCOL, DESCA( CSRC_ ),
     $           NPCOL )
         IBROW = INDXG2P( IB, DESCB( MB_ ), MYPROW, DESCB( RSRC_ ),
     $           NPROW )
         JBCOL = INDXG2P( JB, DESCB( NB_ ), MYPCOL, DESCB( CSRC_ ),
     $           NPCOL )
         ALL_LOCAL = ( DESCA( MB_ ).EQ.DESCB( MB_ ) ) .AND.
     $               ( DESCA( NB_ ).EQ.DESCB( NB_ ) ) .AND.
     $               ( IAROW.EQ.IBROW ) .AND. ( JACOL.EQ.JBCOL ) .AND.
     $               ( ROFFA.EQ.ROFFB ) .AND. ( COFFA.EQ.COFFB )
      ENDIF
      IF( ALL_LOCAL ) THEN
         JSTART = INDXFIRST( N, JB, NB, MYPCOL, DESCB( CSRC_ ), NPCOL )
         HASWORK = ( JB.LE.JSTART ) .AND. ( JSTART.LE.JB+N-1 )
         IF( HASWORK ) THEN
   10       CONTINUE
            IF( JSTART.LE.JB+N-1 ) THEN
               JEND = JSTART - MOD( JSTART-1+NB, NB ) + ( NB-1 )
               JEND = MIN( JB+N-1, JEND )
               JSIZE = JEND - JSTART + 1
               CPROC = INDXG2P( JSTART, NB, MYPCOL, DESCB( CSRC_ ),
     $                 NPCOL )
               ISIZE = 0
               IBEND = IB + M - 1
               IBSTART = INDXFIRST( M, IB, MB, MYPROW, DESCB( RSRC_ ),
     $                   NPROW )
               IF( ( IB.LE.IBSTART ) .AND. ( IBSTART.LE.IBEND ) ) THEN
                  ISIZE = NUMROC2( IBEND-IBSTART+1, IBSTART, MB, MYPROW,
     $                    DESCB( RSRC_ ), NPROW )
               ENDIF
               IF( ISIZE.GE.1 ) THEN
                  CALL DESCINITT( UPLO, IBSTART, JSTART, DESCB, IIB,
     $                            JJB, LOFFSET, DESCNEW )
                  CALL INFOG2L( IIB, JJB, DESCNEW, NPROW, NPCOL, MYPROW,
     $                          MYPCOL, LRINDX, LCINDX, RSRC, CSRC )
                  LDB = DESCNEW( LLD_ )
                  IDXB = ( LOFFSET-1 ) + LRINDX + ( LCINDX-1 )*LDB
                  CALL INFOG2L( IA, JA+( JSTART-JB ), DESCA, NPROW,
     $                          NPCOL, MYPROW, MYPCOL, LRINDX, LCINDX,
     $                          RSRC, CSRC )
                  LDA = DESCA( LLD_ )
                  IDXA = LRINDX + ( LCINDX-1 )*LDA
                  DO 20 J = 1, JSIZE
                     CALL DCOPY( ISIZE, A( IDXA ), 1, B( IDXB ), 1 )
                     IDXA = IDXA + LDA
                     IDXB = IDXB + LDB
   20             CONTINUE
   30             CONTINUE
               ENDIF
* end if (isize >= 1)
               JSTART = MAX( JEND+1, JEND-( NB-1 )+NB*NPCOL )
               GOTO 10
            ENDIF
   40       CONTINUE
* end while
         ENDIF
* end if (haswork)
      ELSE
*
*    Less efficient.
*
         DO 50 JSTART = JB, JB + N - 1
            CALL DESCINITT( UPLO, IB, JSTART, DESCB, IIB, JJB, LOFFSET,
     $                      DESCNEW )
            CALL PDCOPY( M, A, IA, JA+( JSTART-JB ), DESCA, 1,
     $                   B( LOFFSET ), IIB, JJB, DESCNEW, 1 )
   50    CONTINUE
   60    CONTINUE
      ENDIF
      RETURN
      END
